@@ -42,7 +42,7 @@
 #define LTTNG_UST_COMM_MAGIC			0xC57C57C5
 
 /* Version for ABI between liblttng-ust, sessiond, consumerd */
-#define LTTNG_UST_ABI_MAJOR_VERSION		5
+#define LTTNG_UST_ABI_MAJOR_VERSION		6
 #define LTTNG_UST_ABI_MINOR_VERSION		0
 
 enum lttng_ust_instrumentation {
@@ -98,7 +98,7 @@ struct lttng_ust_stream {
 	 */
 } LTTNG_PACKED;
 
-#define LTTNG_UST_EVENT_PADDING1	15
+#define LTTNG_UST_EVENT_PADDING1	16
 #define LTTNG_UST_EVENT_PADDING2	(LTTNG_UST_SYM_NAME_LEN + 32)
 struct lttng_ust_event {
 	enum lttng_ust_instrumentation instrumentation;
@@ -106,7 +106,6 @@ struct lttng_ust_event {
 
 	enum lttng_ust_loglevel_type loglevel_type;
 	int loglevel;	/* value, -1: all */
-	char disabled;
 	char padding[LTTNG_UST_EVENT_PADDING1];
 
 	/* Per instrumentation type configuration */
@@ -140,6 +139,7 @@ enum lttng_ust_context_type {
 	LTTNG_UST_CONTEXT_PROCNAME		= 3,
 	LTTNG_UST_CONTEXT_IP			= 4,
 	LTTNG_UST_CONTEXT_PERF_THREAD_COUNTER	= 5,
+	LTTNG_UST_CONTEXT_CPU_ID		= 6,
 };
 
 struct lttng_ust_perf_counter_ctx {
