@@ -437,7 +437,8 @@ struct lttng_channel_ops {
 			unsigned int switch_timer_interval,
 			unsigned int read_timer_interval,
 			unsigned char *uuid,
-			uint32_t chan_id);
+			uint32_t chan_id,
+			const int *stream_fds, int nr_stream_fds);
 	void (*channel_destroy)(struct lttng_channel *chan);
 	union {
 		void *_deprecated1;
@@ -598,6 +599,7 @@ int lttng_add_vpid_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_pthread_id_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_procname_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_ip_to_ctx(struct lttng_ctx **ctx);
+int lttng_add_cpu_id_to_ctx(struct lttng_ctx **ctx);
 void lttng_context_vtid_reset(void);
 void lttng_context_vpid_reset(void);
 
