@@ -165,8 +165,9 @@ static const char							\
 	},
 
 #undef _ctf_sequence_encoded
-#define _ctf_sequence_encoded(_type, _item, _src,	\
-			_length_type, _src_length, _encoding, _nowrite)	\
+#define _ctf_sequence_encoded(_type, _item, _src,		\
+			_length_type, _src_length, _encoding, _nowrite, \
+			_elem_type_base)			\
 	{							\
 	  .name = #_item,					\
 	  .type =						\
@@ -177,7 +178,7 @@ static const char							\
 			  .sequence =				\
 				{				\
 				  .length_type = __type_integer(_length_type, BYTE_ORDER, 10, none), \
-				  .elem_type = __type_integer(_type, BYTE_ORDER, 10, _encoding), \
+				  .elem_type = __type_integer(_type, BYTE_ORDER, _elem_type_base, _encoding), \
 				},				\
 			},					\
 		},						\
@@ -254,8 +255,8 @@ static void __event_probe__##_provider##___##_name(_TP_ARGS_DATA_PROTO(_args));
 	__event_len += sizeof(_type) * (_length);
 
 #undef _ctf_sequence_encoded
-#define _ctf_sequence_encoded(_type, _item, _src, _length_type,	\
-			_src_length, _encoding, _nowrite)	\
+#define _ctf_sequence_encoded(_type, _item, _src, _length_type,			 \
+			_src_length, _encoding, _nowrite, _elem_type_base)	 \
 	__event_len += lib_ring_buffer_align(__event_len, lttng_alignof(_length_type));   \
 	__event_len += sizeof(_length_type);				       \
 	__event_len += lib_ring_buffer_align(__event_len, lttng_alignof(_type)); \
@@ -391,7 +392,7 @@ size_t __event_get_size__##_provider##___##_name(size_t *__dynamic_len, _TP_ARGS
 
 #undef _ctf_sequence_encoded
 #define _ctf_sequence_encoded(_type, _item, _src, _length_type,		       \
-			_src_length, _encoding, _nowrite)		       \
+			_src_length, _encoding, _nowrite, _elem_type_base)     \
 	{								       \
 		unsigned long __ctf_tmp_ulong = (unsigned long) (_src_length); \
 		const void *__ctf_tmp_ptr = (_src);			       \
@@ -451,8 +452,8 @@ void __event_prepare_filter_stack__##_provider##___##_name(char *__stack_data,\
 	__event_align = _tp_max_t(size_t, __event_align, lttng_alignof(_type));
 
 #undef _ctf_sequence_encoded
-#define _ctf_sequence_encoded(_type, _item, _src, _length_type,	\
-			_src_length, _encoding, _nowrite)	\
+#define _ctf_sequence_encoded(_type, _item, _src, _length_type,		       \
+			_src_length, _encoding, _nowrite, _elem_type_base)     \
 	__event_align = _tp_max_t(size_t, __event_align, lttng_alignof(_length_type));	  \
 	__event_align = _tp_max_t(size_t, __event_align, lttng_alignof(_type));
 
@@ -514,7 +515,7 @@ size_t __event_get_align__##_provider##___##_name(_TP_ARGS_PROTO(_args))      \
 
 #undef _ctf_sequence_encoded
 #define _ctf_sequence_encoded(_type, _item, _src, _length_type,		\
-			_src_length, _encoding, _nowrite)		\
+			_src_length, _encoding, _nowrite, _elem_type_base) \
 	{								\
 		_length_type __tmpl = __stackvar.__dynamic_len[__dynamic_len_idx]; \
 		lib_ring_buffer_align_ctx(&__ctx, lttng_alignof(_length_type));\
