@@ -20,6 +20,7 @@
  */
 
 #define _LGPL_SOURCE
+#define _GNU_SOURCE
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/mman.h>
@@ -51,7 +52,9 @@
 #include "lttng-tracer-core.h"
 #include "compat.h"
 #include "../libringbuffer/tlsfixup.h"
-#include "lttng-ust-baddr.h"
+#include "lttng-ust-statedump.h"
+#include "clock.h"
+#include "../libringbuffer/getcpu.h"
 #include "getenv.h"
 
 /*
@@ -111,8 +114,15 @@ static int lttng_ust_comm_should_quit;
 int ust_lock(void)
 {
 	sigset_t sig_all_blocked, orig_mask;
-	int ret;
+	int ret, oldstate;
 
+	ret = pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &oldstate);
+	if (ret) {
+		ERR("pthread_setcancelstate: %s", strerror(ret));
+	}
+	if (oldstate != PTHREAD_CANCEL_ENABLE) {
+		ERR("pthread_setcancelstate: unexpected oldstate");
+	}
 	sigfillset(&sig_all_blocked);
 	ret = pthread_sigmask(SIG_SETMASK, &sig_all_blocked, &orig_mask);
 	if (ret) {
@@ -140,8 +150,15 @@ int ust_lock(void)
 void ust_lock_nocheck(void)
 {
 	sigset_t sig_all_blocked, orig_mask;
-	int ret;
+	int ret, oldstate;
 
+	ret = pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &oldstate);
+	if (ret) {
+		ERR("pthread_setcancelstate: %s", strerror(ret));
+	}
+	if (oldstate != PTHREAD_CANCEL_ENABLE) {
+		ERR("pthread_setcancelstate: unexpected oldstate");
+	}
 	sigfillset(&sig_all_blocked);
 	ret = pthread_sigmask(SIG_SETMASK, &sig_all_blocked, &orig_mask);
 	if (ret) {
@@ -161,7 +178,7 @@ void ust_lock_nocheck(void)
 void ust_unlock(void)
 {
 	sigset_t sig_all_blocked, orig_mask;
-	int ret;
+	int ret, oldstate;
 
 	sigfillset(&sig_all_blocked);
 	ret = pthread_sigmask(SIG_SETMASK, &sig_all_blocked, &orig_mask);
@@ -174,6 +191,13 @@ void ust_unlock(void)
 	if (ret) {
 		ERR("pthread_sigmask: %s", strerror(ret));
 	}
+	ret = pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &oldstate);
+	if (ret) {
+		ERR("pthread_setcancelstate: %s", strerror(ret));
+	}
+	if (oldstate != PTHREAD_CANCEL_DISABLE) {
+		ERR("pthread_setcancelstate: unexpected oldstate");
+	}
 }
 
 /*
@@ -552,13 +576,13 @@ int handle_message(struct sock_info *sock_info,
 
 	if (ust_lock()) {
 		ret = -LTTNG_UST_ERR_EXITING;
-		goto end;
+		goto error;
 	}
 
 	ops = objd_ops(lum->handle);
 	if (!ops) {
 		ret = -ENOENT;
-		goto end;
+		goto error;
 	}
 
 	switch (lum->cmd) {
@@ -619,12 +643,12 @@ int handle_message(struct sock_info *sock_info,
 				}
 				ret = len;
 				free(bytecode);
-				goto end;
+				goto error;
 			} else {
 				DBG("incorrect filter data message size: %zd", len);
 				ret = -EINVAL;
 				free(bytecode);
-				goto end;
+				goto error;
 			}
 		}
 		bytecode->bc.len = lum->u.filter.data_size;
@@ -684,12 +708,12 @@ int handle_message(struct sock_info *sock_info,
 				}
 				ret = len;
 				free(node);
-				goto end;
+				goto error;
 			} else {
 				DBG("Incorrect exclusion data message size: %zd", len);
 				ret = -EINVAL;
 				free(node);
-				goto end;
+				goto error;
 			}
 		}
 		if (ops->cmd) {
@@ -730,11 +754,11 @@ int handle_message(struct sock_info *sock_info,
 					goto error;
 				}
 				ret = len;
-				goto end;
+				goto error;
 			} else {
 				DBG("incorrect channel data message size: %zd", len);
 				ret = -EINVAL;
-				goto end;
+				goto error;
 			}
 		}
 		args.channel.chan_data = chan_data;
@@ -755,7 +779,7 @@ int handle_message(struct sock_info *sock_info,
 			&args.stream.shm_fd,
 			&args.stream.wakeup_fd);
 		if (ret) {
-			goto end;
+			goto error;
 		}
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
@@ -775,7 +799,6 @@ int handle_message(struct sock_info *sock_info,
 		break;
 	}
 
-end:
 	lur.handle = lum->handle;
 	lur.cmd = lum->cmd;
 	lur.ret_val = ret;
@@ -907,7 +930,12 @@ void cleanup_sock_info(struct sock_info *sock_info, int exiting)
 		long page_size;
 
 		page_size = sysconf(_SC_PAGE_SIZE);
-		if (page_size > 0) {
+		if (page_size <= 0) {
+			if (!page_size) {
+				errno = EINVAL;
+			}
+			PERROR("Error in sysconf(_SC_PAGE_SIZE)");
+		} else {
 			ret = munmap(sock_info->wait_shm_mmap, page_size);
 			if (ret) {
 				ERR("Error unmapping wait shm");
@@ -1089,7 +1117,11 @@ char *get_map_shm(struct sock_info *sock_info)
 	char *wait_shm_mmap;
 
 	page_size = sysconf(_SC_PAGE_SIZE);
-	if (page_size < 0) {
+	if (page_size <= 0) {
+		if (!page_size) {
+			errno = EINVAL;
+		}
+		PERROR("Error in sysconf(_SC_PAGE_SIZE)");
 		goto error;
 	}
 
@@ -1117,8 +1149,6 @@ error:
 static
 void wait_for_sessiond(struct sock_info *sock_info)
 {
-	int ret;
-
 	if (ust_lock()) {
 		goto quit;
 	}
@@ -1134,23 +1164,32 @@ void wait_for_sessiond(struct sock_info *sock_info)
 
 	DBG("Waiting for %s apps sessiond", sock_info->name);
 	/* Wait for futex wakeup */
-	if (uatomic_read((int32_t *) sock_info->wait_shm_mmap) == 0) {
-		ret = futex_async((int32_t *) sock_info->wait_shm_mmap,
-			FUTEX_WAIT, 0, NULL, NULL, 0);
-		if (ret < 0) {
-			if (errno == EFAULT) {
-				wait_poll_fallback = 1;
-				DBG(
+	if (uatomic_read((int32_t *) sock_info->wait_shm_mmap))
+		goto end_wait;
+
+	while (futex_async((int32_t *) sock_info->wait_shm_mmap,
+			FUTEX_WAIT, 0, NULL, NULL, 0)) {
+		switch (errno) {
+		case EWOULDBLOCK:
+			/* Value already changed. */
+			goto end_wait;
+		case EINTR:
+			/* Retry if interrupted by signal. */
+			break;	/* Get out of switch. */
+		case EFAULT:
+			wait_poll_fallback = 1;
+			DBG(
 "Linux kernels 2.6.33 to 3.0 (with the exception of stable versions) "
 "do not support FUTEX_WAKE on read-only memory mappings correctly. "
 "Please upgrade your kernel "
 "(fix is commit 9ea71503a8ed9184d2d0b8ccc4d269d05f7940ae in Linux kernel "
 "mainline). LTTng-UST will use polling mode fallback.");
-				if (ust_debug())
-					PERROR("futex");
-			}
+			if (ust_debug())
+				PERROR("futex");
+			goto end_wait;
 		}
 	}
+end_wait:
 	return;
 
 quit:
@@ -1189,8 +1228,9 @@ restart:
 			 * deals with a killed or broken session daemon.
 			 */
 			sleep(5);
+		} else {
+			has_waited = 1;
 		}
-		has_waited = 1;
 		prev_connect_failed = 0;
 	}
 
@@ -1369,7 +1409,13 @@ restart:
 			print_cmd(lum.cmd, lum.handle);
 			ret = handle_message(sock_info, sock, &lum);
 			if (ret) {
-				ERR("Error handling message for %s socket", sock_info->name);
+				ERR("Error handling message for %s socket",
+					sock_info->name);
+				/*
+				 * Close socket if protocol error is
+				 * detected.
+				 */
+				goto end;
 			}
 			continue;
 		default:
@@ -1447,7 +1493,9 @@ void __attribute__((constructor)) lttng_ust_init(void)
 	 */
 	init_usterr();
 	init_tracepoint();
-	lttng_ust_baddr_statedump_init();
+	lttng_ust_clock_init();
+	lttng_ust_getcpu_init();
+	lttng_ust_statedump_init();
 	lttng_ring_buffer_metadata_client_init();
 	lttng_ring_buffer_client_overwrite_init();
 	lttng_ring_buffer_client_overwrite_rt_init();
@@ -1549,9 +1597,7 @@ static
 void lttng_ust_cleanup(int exiting)
 {
 	cleanup_sock_info(&global_apps, exiting);
-	if (local_apps.allowed) {
-		cleanup_sock_info(&local_apps, exiting);
-	}
+	cleanup_sock_info(&local_apps, exiting);
 	/*
 	 * The teardown in this function all affect data structures
 	 * accessed under the UST lock by the listener thread. This
@@ -1568,7 +1614,7 @@ void lttng_ust_cleanup(int exiting)
 	lttng_ring_buffer_client_overwrite_rt_exit();
 	lttng_ring_buffer_client_overwrite_exit();
 	lttng_ring_buffer_metadata_client_exit();
-	lttng_ust_baddr_statedump_destroy();
+	lttng_ust_statedump_destroy();
 	exit_tracepoint();
 	if (!exiting) {
 		/* Reinitialize values for fork */
