@@ -19,6 +19,7 @@
  */
 
 #define _GNU_SOURCE
+#include <unistd.h>
 #include <urcu/arch.h>
 #include <limits.h>
 
@@ -45,10 +46,15 @@ int lib_ring_buffer_backend_allocate(const struct lttng_ust_lib_ring_buffer_conf
 				     struct lttng_ust_shm_handle *handle,
 				     struct shm_object *shmobj)
 {
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
+	struct channel_backend *chanb;
 	unsigned long subbuf_size, mmap_offset = 0;
 	unsigned long num_subbuf_alloc;
 	unsigned long i;
+	long page_size;
+
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		return -EINVAL;
 
 	subbuf_size = chanb->subbuf_size;
 	num_subbuf_alloc = num_subbuf;
@@ -56,6 +62,11 @@ int lib_ring_buffer_backend_allocate(const struct lttng_ust_lib_ring_buffer_conf
 	if (extra_reader_sb)
 		num_subbuf_alloc++;
 
+	page_size = sysconf(_SC_PAGE_SIZE);
+	if (page_size <= 0) {
+		goto page_size_error;
+	}
+
 	align_shm(shmobj, __alignof__(struct lttng_ust_lib_ring_buffer_backend_pages_shmp));
 	set_shmp(bufb->array, zalloc_shm(shmobj,
 			sizeof(struct lttng_ust_lib_ring_buffer_backend_pages_shmp) * num_subbuf_alloc));
@@ -64,9 +75,9 @@ int lib_ring_buffer_backend_allocate(const struct lttng_ust_lib_ring_buffer_conf
 
 	/*
 	 * This is the largest element (the buffer pages) which needs to
-	 * be aligned on PAGE_SIZE.
+	 * be aligned on page size.
 	 */
-	align_shm(shmobj, PAGE_SIZE);
+	align_shm(shmobj, page_size);
 	set_shmp(bufb->memory_map, zalloc_shm(shmobj,
 			subbuf_size * num_subbuf_alloc));
 	if (caa_unlikely(!shmp(handle, bufb->memory_map)))
@@ -90,8 +101,14 @@ int lib_ring_buffer_backend_allocate(const struct lttng_ust_lib_ring_buffer_conf
 	if (caa_unlikely(!shmp(handle, bufb->buf_wsb)))
 		goto free_array;
 
-	for (i = 0; i < num_subbuf; i++)
-		shmp_index(handle, bufb->buf_wsb, i)->id = subbuffer_id(config, 0, 1, i);
+	for (i = 0; i < num_subbuf; i++) {
+		struct lttng_ust_lib_ring_buffer_backend_subbuffer *sb;
+
+		sb = shmp_index(handle, bufb->buf_wsb, i);
+		if (!sb)
+			goto free_array;
+		sb->id = subbuffer_id(config, 0, 1, i);
+	}
 
 	/* Assign read-side subbuffer table */
 	if (extra_reader_sb)
@@ -102,20 +119,26 @@ int lib_ring_buffer_backend_allocate(const struct lttng_ust_lib_ring_buffer_conf
 
 	/* Assign pages to page index */
 	for (i = 0; i < num_subbuf_alloc; i++) {
+		struct lttng_ust_lib_ring_buffer_backend_pages_shmp *sbp;
+		struct lttng_ust_lib_ring_buffer_backend_pages *pages;
 		struct shm_ref ref;
 
 		ref.index = bufb->memory_map._ref.index;
 		ref.offset = bufb->memory_map._ref.offset;
 		ref.offset += i * subbuf_size;
 
-		set_shmp(shmp(handle, shmp_index(handle, bufb->array, i)->shmp)->p,
-			 ref);
+		sbp = shmp_index(handle, bufb->array, i);
+		if (!sbp)
+			goto free_array;
+		pages = shmp(handle, sbp->shmp);
+		if (!pages)
+			goto free_array;
+		set_shmp(pages->p, ref);
 		if (config->output == RING_BUFFER_MMAP) {
-			shmp(handle, shmp_index(handle, bufb->array, i)->shmp)->mmap_offset = mmap_offset;
+			pages->mmap_offset = mmap_offset;
 			mmap_offset += subbuf_size;
 		}
 	}
-
 	return 0;
 
 free_array:
@@ -123,6 +146,7 @@ free_array:
 memory_map_error:
 	/* bufb->array will be freed by shm teardown */
 array_error:
+page_size_error:
 	return -ENOMEM;
 }
 
@@ -145,17 +169,28 @@ int lib_ring_buffer_backend_create(struct lttng_ust_lib_ring_buffer_backend *buf
 void lib_ring_buffer_backend_reset(struct lttng_ust_lib_ring_buffer_backend *bufb,
 				   struct lttng_ust_shm_handle *handle)
 {
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
-	const struct lttng_ust_lib_ring_buffer_config *config = &chanb->config;
+	struct channel_backend *chanb;
+	const struct lttng_ust_lib_ring_buffer_config *config;
 	unsigned long num_subbuf_alloc;
 	unsigned int i;
 
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		abort();
+	config = &chanb->config;
+
 	num_subbuf_alloc = chanb->num_subbuf;
 	if (chanb->extra_reader_sb)
 		num_subbuf_alloc++;
 
-	for (i = 0; i < chanb->num_subbuf; i++)
-		shmp_index(handle, bufb->buf_wsb, i)->id = subbuffer_id(config, 0, 1, i);
+	for (i = 0; i < chanb->num_subbuf; i++) {
+		struct lttng_ust_lib_ring_buffer_backend_subbuffer *sb;
+
+		sb = shmp_index(handle, bufb->buf_wsb, i);
+		if (!sb)
+			abort();
+		sb->id = subbuffer_id(config, 0, 1, i);
+	}
 	if (chanb->extra_reader_sb)
 		bufb->buf_rsb.id = subbuffer_id(config, 0, 1,
 						num_subbuf_alloc - 1);
@@ -163,10 +198,19 @@ void lib_ring_buffer_backend_reset(struct lttng_ust_lib_ring_buffer_backend *buf
 		bufb->buf_rsb.id = subbuffer_id(config, 0, 1, 0);
 
 	for (i = 0; i < num_subbuf_alloc; i++) {
+		struct lttng_ust_lib_ring_buffer_backend_pages_shmp *sbp;
+		struct lttng_ust_lib_ring_buffer_backend_pages *pages;
+
+		sbp = shmp_index(handle, bufb->array, i);
+		if (!sbp)
+			abort();
+		pages = shmp(handle, sbp->shmp);
+		if (!pages)
+			abort();
 		/* Don't reset mmap_offset */
-		v_set(config, &shmp(handle, shmp_index(handle, bufb->array, i)->shmp)->records_commit, 0);
-		v_set(config, &shmp(handle, shmp_index(handle, bufb->array, i)->shmp)->records_unread, 0);
-		shmp(handle, shmp_index(handle, bufb->array, i)->shmp)->data_size = 0;
+		v_set(config, &pages->records_commit, 0);
+		v_set(config, &pages->records_unread, 0);
+		pages->data_size = 0;
 		/* Don't reset backend page and virt addresses */
 	}
 	/* Don't reset num_pages_per_subbuf, cpu, allocated */
@@ -196,9 +240,10 @@ void channel_backend_reset(struct channel_backend *chanb)
  * @name: channel name
  * @config: client ring buffer configuration
  * @parent: dentry of parent directory, %NULL for root directory
- * @subbuf_size: size of sub-buffers (> PAGE_SIZE, power of 2)
+ * @subbuf_size: size of sub-buffers (> page size, power of 2)
  * @num_subbuf: number of sub-buffers (power of 2)
  * @lttng_ust_shm_handle: shared memory handle
+ * @stream_fds: stream file descriptors.
  *
  * Returns channel pointer if successful, %NULL otherwise.
  *
@@ -212,18 +257,24 @@ int channel_backend_init(struct channel_backend *chanb,
 			 const char *name,
 			 const struct lttng_ust_lib_ring_buffer_config *config,
 			 size_t subbuf_size, size_t num_subbuf,
-			 struct lttng_ust_shm_handle *handle)
+			 struct lttng_ust_shm_handle *handle,
+			 const int *stream_fds)
 {
 	struct channel *chan = caa_container_of(chanb, struct channel, backend);
 	unsigned int i;
 	int ret;
 	size_t shmsize = 0, num_subbuf_alloc;
+	long page_size;
 
 	if (!name)
 		return -EPERM;
 
+	page_size = sysconf(_SC_PAGE_SIZE);
+	if (page_size <= 0) {
+		return -ENOMEM;
+	}
 	/* Check that the subbuffer size is larger than a page. */
-	if (subbuf_size < PAGE_SIZE)
+	if (subbuf_size < page_size)
 		return -EINVAL;
 
 	/*
@@ -234,6 +285,12 @@ int channel_backend_init(struct channel_backend *chanb,
 		return -EINVAL;
 	if (!num_subbuf || (num_subbuf & (num_subbuf - 1)))
 		return -EINVAL;
+	/*
+	 * Overwrite mode buffers require at least 2 subbuffers per
+	 * buffer.
+	 */
+	if (config->mode == RING_BUFFER_OVERWRITE && num_subbuf < 2)
+		return -EINVAL;
 
 	ret = subbuffer_id_check_index(config, num_subbuf);
 	if (ret)
@@ -260,7 +317,7 @@ int channel_backend_init(struct channel_backend *chanb,
 	num_subbuf_alloc = num_subbuf + 1;
 	shmsize += offset_align(shmsize, __alignof__(struct lttng_ust_lib_ring_buffer_backend_pages_shmp));
 	shmsize += sizeof(struct lttng_ust_lib_ring_buffer_backend_pages_shmp) * num_subbuf_alloc;
-	shmsize += offset_align(shmsize, PAGE_SIZE);
+	shmsize += offset_align(shmsize, page_size);
 	shmsize += subbuf_size * num_subbuf_alloc;
 	shmsize += offset_align(shmsize, __alignof__(struct lttng_ust_lib_ring_buffer_backend_pages));
 	shmsize += sizeof(struct lttng_ust_lib_ring_buffer_backend_pages) * num_subbuf_alloc;
@@ -281,7 +338,7 @@ int channel_backend_init(struct channel_backend *chanb,
 			struct shm_object *shmobj;
 
 			shmobj = shm_object_table_alloc(handle->table, shmsize,
-					SHM_OBJECT_SHM);
+					SHM_OBJECT_SHM, stream_fds[i]);
 			if (!shmobj)
 				goto end;
 			align_shm(shmobj, __alignof__(struct lttng_ust_lib_ring_buffer));
@@ -300,7 +357,7 @@ int channel_backend_init(struct channel_backend *chanb,
 		struct lttng_ust_lib_ring_buffer *buf;
 
 		shmobj = shm_object_table_alloc(handle->table, shmsize,
-					SHM_OBJECT_SHM);
+					SHM_OBJECT_SHM, stream_fds[0]);
 		if (!shmobj)
 			goto end;
 		align_shm(shmobj, __alignof__(struct lttng_ust_lib_ring_buffer));
@@ -349,12 +406,17 @@ void channel_backend_free(struct channel_backend *chanb,
 size_t lib_ring_buffer_read(struct lttng_ust_lib_ring_buffer_backend *bufb, size_t offset,
 			    void *dest, size_t len, struct lttng_ust_shm_handle *handle)
 {
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
-	const struct lttng_ust_lib_ring_buffer_config *config = &chanb->config;
+	struct channel_backend *chanb;
+	const struct lttng_ust_lib_ring_buffer_config *config;
 	ssize_t orig_len;
 	struct lttng_ust_lib_ring_buffer_backend_pages_shmp *rpages;
 	unsigned long sb_bindex, id;
+	void *src;
 
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		return 0;
+	config = &chanb->config;
 	orig_len = len;
 	offset &= chanb->buf_size - 1;
 
@@ -370,7 +432,11 @@ size_t lib_ring_buffer_read(struct lttng_ust_lib_ring_buffer_backend *bufb, size
 	CHAN_WARN_ON(chanb, offset >= chanb->buf_size);
 	CHAN_WARN_ON(chanb, config->mode == RING_BUFFER_OVERWRITE
 		     && subbuffer_id_is_noref(config, id));
-	memcpy(dest, shmp_index(handle, shmp(handle, rpages->shmp)->p, offset & (chanb->subbuf_size - 1)), len);
+	src = shmp_index(handle, shmp(handle, rpages->shmp)->p,
+			offset & (chanb->subbuf_size - 1));
+	if (caa_unlikely(!src))
+		return 0;
+	memcpy(dest, src, len);
 	return orig_len;
 }
 
@@ -388,13 +454,17 @@ size_t lib_ring_buffer_read(struct lttng_ust_lib_ring_buffer_backend *bufb, size
 int lib_ring_buffer_read_cstr(struct lttng_ust_lib_ring_buffer_backend *bufb, size_t offset,
 			      void *dest, size_t len, struct lttng_ust_shm_handle *handle)
 {
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
-	const struct lttng_ust_lib_ring_buffer_config *config = &chanb->config;
+	struct channel_backend *chanb;
+	const struct lttng_ust_lib_ring_buffer_config *config;
 	ssize_t string_len, orig_offset;
 	char *str;
 	struct lttng_ust_lib_ring_buffer_backend_pages_shmp *rpages;
 	unsigned long sb_bindex, id;
 
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		return -EINVAL;
+	config = &chanb->config;
 	if (caa_unlikely(!len))
 		return -EINVAL;
 	offset &= chanb->buf_size - 1;
@@ -410,6 +480,8 @@ int lib_ring_buffer_read_cstr(struct lttng_ust_lib_ring_buffer_backend *bufb, si
 	CHAN_WARN_ON(chanb, config->mode == RING_BUFFER_OVERWRITE
 		     && subbuffer_id_is_noref(config, id));
 	str = shmp_index(handle, shmp(handle, rpages->shmp)->p, offset & (chanb->subbuf_size - 1));
+	if (caa_unlikely(!str))
+		return -EINVAL;
 	string_len = strnlen(str, len);
 	if (dest && len) {
 		memcpy(dest, str, string_len);
@@ -434,10 +506,14 @@ void *lib_ring_buffer_read_offset_address(struct lttng_ust_lib_ring_buffer_backe
 					  struct lttng_ust_shm_handle *handle)
 {
 	struct lttng_ust_lib_ring_buffer_backend_pages_shmp *rpages;
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
-	const struct lttng_ust_lib_ring_buffer_config *config = &chanb->config;
+	struct channel_backend *chanb;
+	const struct lttng_ust_lib_ring_buffer_config *config;
 	unsigned long sb_bindex, id;
 
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		return NULL;
+	config = &chanb->config;
 	offset &= chanb->buf_size - 1;
 	id = bufb->buf_rsb.id;
 	sb_bindex = subbuffer_id_get_index(config, id);
@@ -463,13 +539,21 @@ void *lib_ring_buffer_offset_address(struct lttng_ust_lib_ring_buffer_backend *b
 {
 	size_t sbidx;
 	struct lttng_ust_lib_ring_buffer_backend_pages_shmp *rpages;
-	struct channel_backend *chanb = &shmp(handle, bufb->chan)->backend;
-	const struct lttng_ust_lib_ring_buffer_config *config = &chanb->config;
+	struct channel_backend *chanb;
+	const struct lttng_ust_lib_ring_buffer_config *config;
 	unsigned long sb_bindex, id;
+	struct lttng_ust_lib_ring_buffer_backend_subbuffer *sb;
 
+	chanb = &shmp(handle, bufb->chan)->backend;
+	if (!chanb)
+		return NULL;
+	config = &chanb->config;
 	offset &= chanb->buf_size - 1;
 	sbidx = offset >> chanb->subbuf_size_order;
-	id = shmp_index(handle, bufb->buf_wsb, sbidx)->id;
+	sb = shmp_index(handle, bufb->buf_wsb, sbidx);
+	if (!sb)
+		return NULL;
+	id = sb->id;
 	sb_bindex = subbuffer_id_get_index(config, id);
 	rpages = shmp_index(handle, bufb->array, sb_bindex);
 	CHAN_WARN_ON(chanb, config->mode == RING_BUFFER_OVERWRITE
